@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 2008 Nuanti Ltd.
  * Copyright (C) 2009 Igalia S.L.
+ * Copyright (C) 2009 Jan Alonzo
  *
  * Portions from Mozilla a11y, copyright as follows:
  *
@@ -59,7 +60,7 @@ using namespace WebCore;
 
 static AccessibilityObject* fallbackObject()
 {
-    static AXObjectCache* fallbackCache = new AXObjectCache();
+    static AXObjectCache* fallbackCache = new AXObjectCache;
     static AccessibilityObject* object = 0;
     if (!object) {
         // FIXME: using fallbackCache->getOrCreate(ListBoxOptionRole) is a hack
@@ -555,7 +556,7 @@ static PangoLayout* getPangoLayoutForAtk(AtkText* textObject)
     HostWindow* hostWindow = coreObject->document()->view()->hostWindow();
     if (!hostWindow)
         return 0;
-    PlatformWidget webView = hostWindow->platformWindow();
+    PlatformPageClient webView = hostWindow->platformPageClient();
     if (!webView)
         return 0;
 
@@ -564,7 +565,7 @@ static PangoLayout* getPangoLayoutForAtk(AtkText* textObject)
     AccessibilityRenderObject* accObject = static_cast<AccessibilityRenderObject*>(coreObject);
     if (!accObject)
         return 0;
-    RenderText* renderText = static_cast<RenderText*>(accObject->renderer());
+    RenderText* renderText = toRenderText(accObject->renderer());
     if (!renderText)
         return 0;
 
@@ -1078,4 +1079,16 @@ void webkit_accessible_detach(WebKitAccessible* accessible)
     accessible->m_object = fallbackObject();
 }
 
+AtkObject* webkit_accessible_get_focused_element(WebKitAccessible* accessible)
+{
+    if (!accessible->m_object)
+        return 0;
+
+    RefPtr<AccessibilityObject> focusedObj = accessible->m_object->focusedUIElement();
+    if (!focusedObj)
+        return 0;
+
+    return focusedObj->wrapper();
+}
+
 #endif // HAVE(ACCESSIBILITY)
