@@ -32,9 +32,10 @@
 #if ENABLE(DATABASE)
 
 #include "Frame.h"
-#include "ScriptController.h"
+#include "JSDOMGlobalObject.h"
 #include "JSSQLTransaction.h"
 #include "Page.h"
+#include "ScriptController.h"
 #include <runtime/JSLock.h>
 #include <wtf/MainThread.h>
 #include <wtf/RefCountedLeakCounter.h>
@@ -47,25 +48,28 @@ using namespace JSC;
 static WTF::RefCountedLeakCounter counter("JSCustomSQLTransactionCallback");
 #endif
 
-// We have to clean up the data on the main thread for two reasons:
-//
-//     1) Can't deref a Frame on a non-main thread.
-//     2) Unprotecting the JSObject on a non-main thread would register that thread
-//        for JavaScript garbage collection, which could unnecessarily slow things down.
+// We have to clean up the data on the main thread because unprotecting the 
+// JSObject on a non-main thread would register that thread for JavaScript
+// garbage collection, which could unnecessarily slow things down.
 
 class JSCustomSQLTransactionCallback::Data {
 public:
-    Data(JSObject* callback, Frame* frame) : m_callback(callback), m_frame(frame) { }
+    Data(JSObject* callback, JSDOMGlobalObject* globalObject)
+        : m_callback(callback)
+        , m_globalObject(globalObject)
+    {
+    }
+
     JSObject* callback() { return m_callback; }
-    Frame* frame() { return m_frame.get(); }
+    JSDOMGlobalObject* globalObject() { return m_globalObject.get(); }
 
 private:
     ProtectedPtr<JSObject> m_callback;
-    RefPtr<Frame> m_frame;
+    ProtectedPtr<JSDOMGlobalObject> m_globalObject;
 };
 
-JSCustomSQLTransactionCallback::JSCustomSQLTransactionCallback(JSObject* callback, Frame* frame)
-    : m_data(new Data(callback, frame))
+JSCustomSQLTransactionCallback::JSCustomSQLTransactionCallback(JSObject* callback, JSDOMGlobalObject* globalObject)
+    : m_data(new Data(callback, globalObject))
 {
 #ifndef NDEBUG
     counter.increment();
@@ -90,13 +94,9 @@ void JSCustomSQLTransactionCallback::handleEvent(SQLTransaction* transaction, bo
 {
     ASSERT(m_data);
     ASSERT(m_data->callback());
-    ASSERT(m_data->frame());
+    ASSERT(m_data->globalObject());
 
-    if (!m_data->frame()->script()->isEnabled())
-        return;
-
-    // FIXME: This is likely the wrong globalObject (for prototype chains at least)
-    JSGlobalObject* globalObject = m_data->frame()->script()->globalObject();
+    JSDOMGlobalObject* globalObject = m_data->globalObject();
     ExecState* exec = globalObject->globalExec();
         
     JSC::JSLock lock(SilenceAssertionsOnly);
