@@ -46,8 +46,6 @@ private:
     virtual void doApply();
     virtual EditAction editingAction() const { return m_typeOfAction == Indent ? EditActionIndent : EditActionOutdent; }
 
-    // FIXME: Does this belong in htmlediting.cpp?
-    bool isAtUnsplittableElement(const Position&) const;
     void appendParagraphIntoNode(const VisiblePosition& start, const VisiblePosition& end, Node* newParent);
     void removeUnnecessaryLineBreakAt(const Position& endOfParagraph);
 
