@@ -57,6 +57,8 @@ public:
     void setOnbeforeunload(PassRefPtr<EventListener>);
     EventListener* onmessage() const;
     void setOnmessage(PassRefPtr<EventListener>);
+    EventListener* onhashchange() const;
+    void setOnhashchange(PassRefPtr<EventListener>);
     EventListener* onoffline() const;
     void setOnoffline(PassRefPtr<EventListener>);
     EventListener* ononline() const;
