@@ -51,7 +51,7 @@ void GeneratedImage::drawPattern(GraphicsContext* context, const FloatRect& srcR
                                  const FloatPoint& phase, CompositeOperator compositeOp, const FloatRect& destRect)
 {
     // Create a BitmapImage and call drawPattern on it.
-    OwnPtr<ImageBuffer> imageBuffer = ImageBuffer::create(m_size, false);
+    OwnPtr<ImageBuffer> imageBuffer = ImageBuffer::create(m_size);
     ASSERT(imageBuffer.get());
     
     // Fill with the gradient.
