@@ -165,7 +165,7 @@ void Image::drawPattern(GraphicsContext* context, const FloatRect& tileRect, con
     context->save();
 
     IntRect imageSize = enclosingIntRect(tileRect);
-    OwnPtr<ImageBuffer> imageSurface = ImageBuffer::create(imageSize.size(), false);
+    OwnPtr<ImageBuffer> imageSurface = ImageBuffer::create(imageSize.size());
 
     if (!imageSurface)
         return;
