@@ -24,7 +24,6 @@
 #if ENABLE(VIDEO)
 
 #include "MediaPlayerPrivateGStreamer.h"
-#include "VideoSinkGStreamer.h"
 
 #include "CString.h"
 #include "GraphicsContext.h"
@@ -34,10 +33,10 @@
 #include "MediaPlayer.h"
 #include "NotImplemented.h"
 #include "ScrollView.h"
+#include "VideoSinkGStreamer.h"
 #include "Widget.h"
-#include <wtf/GOwnPtr.h>
+#include "TimeRanges.h"
 
-#include <gdk/gdkx.h>
 #include <gst/base/gstbasesrc.h>
 #include <gst/gst.h>
 #include <gst/interfaces/mixer.h>
@@ -45,6 +44,7 @@
 #include <gst/video/video.h>
 #include <limits>
 #include <math.h>
+#include <wtf/GOwnPtr.h>
 
 using namespace std;
 
@@ -52,27 +52,31 @@ namespace WebCore {
 
 gboolean mediaPlayerPrivateErrorCallback(GstBus* bus, GstMessage* message, gpointer data)
 {
-    if (GST_MESSAGE_TYPE(message) == GST_MESSAGE_ERROR)
-    {
+    if (GST_MESSAGE_TYPE(message) == GST_MESSAGE_ERROR) {
         GOwnPtr<GError> err;
         GOwnPtr<gchar> debug;
 
         gst_message_parse_error(message, &err.outPtr(), &debug.outPtr());
-        if (err->code == 3) {
-            LOG_VERBOSE(Media, "File not found");
-            MediaPlayerPrivate* mp = reinterpret_cast<MediaPlayerPrivate*>(data);
-            if (mp)
-                mp->loadingFailed();
-        } else
-            LOG_VERBOSE(Media, "Error: %d, %s", err->code,  err->message);
+        LOG_VERBOSE(Media, "Error: %d, %s", err->code,  err->message);
+
+        MediaPlayer::NetworkState error = MediaPlayer::Empty;
+        if (err->domain == GST_CORE_ERROR || err->domain == GST_LIBRARY_ERROR)
+            error = MediaPlayer::DecodeError;
+        else if (err->domain == GST_RESOURCE_ERROR)
+            error = MediaPlayer::FormatError;
+        else if (err->domain == GST_STREAM_ERROR)
+            error = MediaPlayer::NetworkError;
+
+        MediaPlayerPrivate* mp = reinterpret_cast<MediaPlayerPrivate*>(data);
+        if (mp)
+            mp->loadingFailed(error);
     }
     return true;
 }
 
 gboolean mediaPlayerPrivateEOSCallback(GstBus* bus, GstMessage* message, gpointer data)
 {
-    if (GST_MESSAGE_TYPE(message) == GST_MESSAGE_EOS)
-    {
+    if (GST_MESSAGE_TYPE(message) == GST_MESSAGE_EOS) {
         LOG_VERBOSE(Media, "End of Stream");
         MediaPlayerPrivate* mp = reinterpret_cast<MediaPlayerPrivate*>(data);
         mp->didEnd();
@@ -82,8 +86,7 @@ gboolean mediaPlayerPrivateEOSCallback(GstBus* bus, GstMessage* message, gpointe
 
 gboolean mediaPlayerPrivateStateCallback(GstBus* bus, GstMessage* message, gpointer data)
 {
-    if (GST_MESSAGE_TYPE(message) == GST_MESSAGE_STATE_CHANGED)
-    {
+    if (GST_MESSAGE_TYPE(message) == GST_MESSAGE_STATE_CHANGED) {
         MediaPlayerPrivate* mp = reinterpret_cast<MediaPlayerPrivate*>(data);
         mp->updateStates();
     }
@@ -92,8 +95,7 @@ gboolean mediaPlayerPrivateStateCallback(GstBus* bus, GstMessage* message, gpoin
 
 gboolean mediaPlayerPrivateBufferingCallback(GstBus* bus, GstMessage* message, gpointer data)
 {
-    if (GST_MESSAGE_TYPE(message) == GST_MESSAGE_BUFFERING)
-    {
+    if (GST_MESSAGE_TYPE(message) == GST_MESSAGE_BUFFERING) {
         gint percent = 0;
         gst_message_parse_buffering(message, &percent);
         LOG_VERBOSE(Media, "Buffering %d", percent);
@@ -106,8 +108,8 @@ static void mediaPlayerPrivateRepaintCallback(WebKitVideoSink*, MediaPlayerPriva
     playerPrivate->repaint();
 }
 
-MediaPlayerPrivateInterface* MediaPlayerPrivate::create(MediaPlayer* player) 
-{ 
+MediaPlayerPrivateInterface* MediaPlayerPrivate::create(MediaPlayer* player)
+{
     return new MediaPlayerPrivate(player);
 }
 
@@ -117,6 +119,8 @@ void MediaPlayerPrivate::registerMediaEngine(MediaEngineRegistrar registrar)
         registrar(create, getSupportedTypes, supportsType);
 }
 
+static bool gstInitialized = false;
+
 MediaPlayerPrivate::MediaPlayerPrivate(MediaPlayer* player)
     : m_player(player)
     , m_playBin(0)
@@ -124,7 +128,6 @@ MediaPlayerPrivate::MediaPlayerPrivate(MediaPlayer* player)
     , m_source(0)
     , m_rate(1.0f)
     , m_endTime(numeric_limits<float>::infinity())
-    , m_isEndReached(false)
     , m_volume(0.5f)
     , m_networkState(MediaPlayer::Empty)
     , m_readyState(MediaPlayer::HaveNothing)
@@ -132,12 +135,13 @@ MediaPlayerPrivate::MediaPlayerPrivate(MediaPlayer* player)
     , m_isStreaming(false)
     , m_size(IntSize())
     , m_visible(true)
+    , m_paused(true)
+    , m_seeking(false)
+    , m_errorOccured(false)
 {
-
-    static bool gstInitialized = false;
     // FIXME: We should pass the arguments from the command line
     if (!gstInitialized) {
-        gst_init(0, NULL);
+        gst_init(0, 0);
         gstInitialized = true;
     }
 
@@ -145,6 +149,12 @@ MediaPlayerPrivate::MediaPlayerPrivate(MediaPlayer* player)
     m_surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, 640, 480);
 }
 
+static gboolean idleUnref(gpointer data)
+{
+    g_object_unref(reinterpret_cast<GObject*>(data));
+    return FALSE;
+}
+
 MediaPlayerPrivate::~MediaPlayerPrivate()
 {
     if (m_surface)
@@ -154,6 +164,18 @@ MediaPlayerPrivate::~MediaPlayerPrivate()
         gst_element_set_state(m_playBin, GST_STATE_NULL);
         gst_object_unref(GST_OBJECT(m_playBin));
     }
+
+    // FIXME: We should find a better way to handle the lifetime of this object; this is
+    // needed because the object is sometimes being destroyed inbetween a call to
+    // webkit_video_sink_render, and the idle it schedules. Adding a ref in
+    // webkit_video_sink_render that would be balanced by the idle is not an option,
+    // because in some cases the destruction of the sink may happen in time for the idle
+    // to be removed from the queue, so it may not run. It would also cause lots of ref
+    // counting churn (render/idle are called many times). This is an ugly race.
+    if (m_videoSink) {
+        g_idle_add(idleUnref, m_videoSink);
+        m_videoSink = 0;
+    }
 }
 
 void MediaPlayerPrivate::load(const String& url)
@@ -175,20 +197,13 @@ void MediaPlayerPrivate::load(const String& url)
 void MediaPlayerPrivate::play()
 {
     LOG_VERBOSE(Media, "Play");
-    // When end reached, rewind for Test video-seek-past-end-playing
-    if (m_isEndReached)
-        seek(0);
-    m_isEndReached = false;
-
     gst_element_set_state(m_playBin, GST_STATE_PLAYING);
-    m_startedPlaying = true;
 }
 
 void MediaPlayerPrivate::pause()
 {
     LOG_VERBOSE(Media, "Pause");
     gst_element_set_state(m_playBin, GST_STATE_PAUSED);
-    m_startedPlaying = false;
 }
 
 float MediaPlayerPrivate::duration() const
@@ -196,6 +211,9 @@ float MediaPlayerPrivate::duration() const
     if (!m_playBin)
         return 0.0;
 
+    if (m_errorOccured)
+        return 0.0;
+
     GstFormat timeFormat = GST_FORMAT_TIME;
     gint64 timeLength = 0;
 
@@ -206,7 +224,6 @@ float MediaPlayerPrivate::duration() const
     // See https://bugs.webkit.org/show_bug.cgi?id=24639.
     if (!gst_element_query_duration(m_playBin, &timeFormat, &timeLength) || timeLength <= 0) {
         LOG_VERBOSE(Media, "Time duration query failed.");
-        m_isStreaming = true;
         return numeric_limits<float>::infinity();
     }
 
@@ -220,22 +237,24 @@ float MediaPlayerPrivate::currentTime() const
 {
     if (!m_playBin)
         return 0;
-    // Necessary as sometimes, gstreamer return 0:00 at the EOS
-    if (m_isEndReached)
-        return m_endTime;
 
-    float ret;
+    if (m_errorOccured)
+        return 0;
+
+    float ret = 0.0;
 
     GstQuery* query = gst_query_new_position(GST_FORMAT_TIME);
-    if (gst_element_query(m_playBin, query)) {
-        gint64 position;
-        gst_query_parse_position(query, NULL, &position);
-        ret = (float) (position / 1000000000.0);
-        LOG_VERBOSE(Media, "Position %" GST_TIME_FORMAT, GST_TIME_ARGS(position));
-    } else {
+    if (!gst_element_query(m_playBin, query)) {
         LOG_VERBOSE(Media, "Position query failed...");
-        ret = 0.0;
+        gst_query_unref(query);
+        return ret;
     }
+
+    gint64 position;
+    gst_query_parse_position(query, 0, &position);
+    ret = (float) (position / 1000000000.0);
+    LOG_VERBOSE(Media, "Position %" GST_TIME_FORMAT, GST_TIME_ARGS(position));
+
     gst_query_unref(query);
 
     return ret;
@@ -251,35 +270,23 @@ void MediaPlayerPrivate::seek(float time)
     if (m_isStreaming)
         return;
 
+    if (m_errorOccured)
+        return;
+
     LOG_VERBOSE(Media, "Seek: %" GST_TIME_FORMAT, GST_TIME_ARGS(sec));
-    // FIXME: What happens when the seeked position is not available?
     if (!gst_element_seek( m_playBin, m_rate,
             GST_FORMAT_TIME,
             (GstSeekFlags)(GST_SEEK_FLAG_FLUSH),
             GST_SEEK_TYPE_SET, sec,
             GST_SEEK_TYPE_NONE, GST_CLOCK_TIME_NONE))
         LOG_VERBOSE(Media, "Seek to %f failed", time);
+    else
+        m_seeking = true;
 }
 
 void MediaPlayerPrivate::setEndTime(float time)
 {
-    if (!m_playBin)
-        return;
-    if (m_isStreaming)
-        return;
-    if (m_endTime != time) {
-        m_endTime = time;
-        GstClockTime start = (GstClockTime)(currentTime() * GST_SECOND);
-        GstClockTime end   = (GstClockTime)(time * GST_SECOND);
-        LOG_VERBOSE(Media, "setEndTime: %" GST_TIME_FORMAT, GST_TIME_ARGS(end));
-        // FIXME: What happens when the seeked position is not available?
-        if (!gst_element_seek(m_playBin, m_rate,
-                GST_FORMAT_TIME,
-                (GstSeekFlags)(GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_ACCURATE),
-                GST_SEEK_TYPE_SET, start,
-                GST_SEEK_TYPE_SET, end ))
-            LOG_VERBOSE(Media, "Seek to %f failed", time);
-    }
+    notImplemented();
 }
 
 void MediaPlayerPrivate::startEndPointTimerIfNeeded()
@@ -299,12 +306,12 @@ void MediaPlayerPrivate::endPointTimerFired(Timer<MediaPlayerPrivate>*)
 
 bool MediaPlayerPrivate::paused() const
 {
-    return !m_startedPlaying;
+    return m_paused;
 }
 
 bool MediaPlayerPrivate::seeking() const
 {
-    return false;
+    return m_seeking;
 }
 
 // Returns the size of the video
@@ -330,6 +337,14 @@ bool MediaPlayerPrivate::hasVideo() const
     return currentVideo > -1;
 }
 
+bool MediaPlayerPrivate::hasAudio() const
+{
+    gint currentAudio = -1;
+    if (m_playBin)
+        g_object_get(G_OBJECT(m_playBin), "current-audio", &currentAudio, NULL);
+    return currentAudio > -1;
+}
+
 void MediaPlayerPrivate::setVolume(float volume)
 {
     m_volume = volume;
@@ -345,9 +360,9 @@ void MediaPlayerPrivate::setMuted(bool b)
     if (b) {
         g_object_get(G_OBJECT(m_playBin), "volume", &m_volume, NULL);
         g_object_set(G_OBJECT(m_playBin), "volume", (double)0.0, NULL);
-    } else {
+    } else
         g_object_set(G_OBJECT(m_playBin), "volume", m_volume, NULL);
-    }
+
 }
 
 void MediaPlayerPrivate::setRate(float rate)
@@ -356,17 +371,13 @@ void MediaPlayerPrivate::setRate(float rate)
         gst_element_set_state(m_playBin, GST_STATE_PAUSED);
         return;
     }
+
     if (m_isStreaming)
         return;
 
     m_rate = rate;
     LOG_VERBOSE(Media, "Set Rate to %f", rate);
-    if (!gst_element_seek(m_playBin, rate,
-            GST_FORMAT_TIME,
-            (GstSeekFlags)(GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_ACCURATE),
-            GST_SEEK_TYPE_SET, (GstClockTime) (currentTime() * GST_SECOND),
-            GST_SEEK_TYPE_SET, (GstClockTime) (m_endTime * GST_SECOND)))
-        LOG_VERBOSE(Media, "Set Rate to %f failed", rate);
+    seek(currentTime());
 }
 
 int MediaPlayerPrivate::dataRate() const
@@ -385,16 +396,20 @@ MediaPlayer::ReadyState MediaPlayerPrivate::readyState() const
     return m_readyState;
 }
 
-float MediaPlayerPrivate::maxTimeBuffered() const
+PassRefPtr<TimeRanges> MediaPlayerPrivate::buffered() const
 {
-    notImplemented();
-    LOG_VERBOSE(Media, "maxTimeBuffered");
-    // rtsp streams are not buffered
-    return m_isStreaming ? 0 : maxTimeLoaded();
+    RefPtr<TimeRanges> timeRanges = TimeRanges::create();
+    float loaded = maxTimeLoaded();
+    if (!m_errorOccured && !m_isStreaming && loaded > 0)
+        timeRanges->add(0, loaded);
+    return timeRanges.release();
 }
 
 float MediaPlayerPrivate::maxTimeSeekable() const
 {
+    if (m_errorOccured)
+        return 0.0;
+
     // TODO
     LOG_VERBOSE(Media, "maxTimeSeekable");
     if (m_isStreaming)
@@ -405,6 +420,9 @@ float MediaPlayerPrivate::maxTimeSeekable() const
 
 float MediaPlayerPrivate::maxTimeLoaded() const
 {
+    if (m_errorOccured)
+        return 0.0;
+
     // TODO
     LOG_VERBOSE(Media, "maxTimeLoaded");
     notImplemented();
@@ -421,29 +439,30 @@ unsigned MediaPlayerPrivate::bytesLoaded() const
     float maxTime = maxTimeLoaded();
     if (!dur)
         return 0;*/
+
     return 1;//totalBytes() * maxTime / dur;
 }
 
 bool MediaPlayerPrivate::totalBytesKnown() const
 {
-    notImplemented();
     LOG_VERBOSE(Media, "totalBytesKnown");
     return totalBytes() > 0;
 }
 
 unsigned MediaPlayerPrivate::totalBytes() const
 {
-    notImplemented();
     LOG_VERBOSE(Media, "totalBytes");
-    if (!m_playBin)
-        return 0;
-
     if (!m_source)
         return 0;
 
-    // Do something with m_source to get the total bytes of the media
+    if (m_errorOccured)
+        return 0;
 
-    return 100;
+    GstFormat fmt = GST_FORMAT_BYTES;
+    gint64 length = 0;
+    gst_element_query_duration(m_source, &fmt, &length);
+
+    return length;
 }
 
 void MediaPlayerPrivate::cancelLoad()
@@ -457,18 +476,22 @@ void MediaPlayerPrivate::updateStates()
     // the state of GStreamer, therefore, when in PAUSED state,
     // we are sure we can display the first frame and go to play
 
+    if (!m_playBin)
+        return;
+
+    if (m_errorOccured)
+        return;
+
     MediaPlayer::NetworkState oldNetworkState = m_networkState;
     MediaPlayer::ReadyState oldReadyState = m_readyState;
     GstState state;
     GstState pending;
 
-    if (!m_playBin)
-        return;
-
-    GstStateChangeReturn ret = gst_element_get_state (m_playBin,
+    GstStateChangeReturn ret = gst_element_get_state(m_playBin,
         &state, &pending, 250 * GST_NSECOND);
 
-    switch(ret) {
+    bool shouldUpdateAfterSeek = false;
+    switch (ret) {
     case GST_STATE_CHANGE_SUCCESS:
         LOG_VERBOSE(Media, "State: %s, pending: %s",
             gst_element_state_get_name(state),
@@ -476,14 +499,24 @@ void MediaPlayerPrivate::updateStates()
 
         if (state == GST_STATE_READY) {
             m_readyState = MediaPlayer::HaveEnoughData;
-        } else if (state == GST_STATE_PAUSED) {
+        } else if (state == GST_STATE_PAUSED)
             m_readyState = MediaPlayer::HaveEnoughData;
+
+        if (state == GST_STATE_PLAYING)
+            m_paused = false;
+        else
+            m_paused = true;
+
+        if (m_seeking) {
+            shouldUpdateAfterSeek = true;
+            m_seeking = false;
         }
+
         m_networkState = MediaPlayer::Loaded;
 
         g_object_get(m_playBin, "source", &m_source, NULL);
         if (!m_source)
-            LOG_VERBOSE(Media, "m_source is NULL");
+            LOG_VERBOSE(Media, "m_source is 0");
         break;
     case GST_STATE_CHANGE_ASYNC:
         LOG_VERBOSE(Media, "Async: State: %s, pending: %s",
@@ -491,16 +524,22 @@ void MediaPlayerPrivate::updateStates()
             gst_element_state_get_name(pending));
         // Change in progress
         return;
-        break;
+    case GST_STATE_CHANGE_FAILURE:
+        LOG_VERBOSE(Media, "Failure: State: %s, pending: %s",
+            gst_element_state_get_name(state),
+            gst_element_state_get_name(pending));
+        // Change failed
+        return;
     case GST_STATE_CHANGE_NO_PREROLL:
         LOG_VERBOSE(Media, "No preroll: State: %s, pending: %s",
             gst_element_state_get_name(state),
             gst_element_state_get_name(pending));
+
         if (state == GST_STATE_READY) {
             m_readyState = MediaPlayer::HaveFutureData;
-        } else if (state == GST_STATE_PAUSED) {
+        } else if (state == GST_STATE_PAUSED)
             m_readyState = MediaPlayer::HaveCurrentData;
-        }
+
         m_networkState = MediaPlayer::Loading;
         break;
     default:
@@ -511,6 +550,9 @@ void MediaPlayerPrivate::updateStates()
     if (seeking())
         m_readyState = MediaPlayer::HaveNothing;
 
+    if (shouldUpdateAfterSeek)
+        timeChanged();
+
     if (m_networkState != oldNetworkState) {
         LOG_VERBOSE(Media, "Network State Changed from %u to %u",
             oldNetworkState, m_networkState);
@@ -551,15 +593,14 @@ void MediaPlayerPrivate::volumeChanged()
 
 void MediaPlayerPrivate::didEnd()
 {
-    m_isEndReached = true;
-    pause();
     timeChanged();
 }
 
-void MediaPlayerPrivate::loadingFailed()
+void MediaPlayerPrivate::loadingFailed(MediaPlayer::NetworkState error)
 {
-    if (m_networkState != MediaPlayer::NetworkError) {
-        m_networkState = MediaPlayer::NetworkError;
+    m_errorOccured = true;
+    if (m_networkState != error) {
+        m_networkState = error;
         m_player->networkStateChanged();
     }
     if (m_readyState != MediaPlayer::HaveNothing) {
@@ -603,24 +644,94 @@ void MediaPlayerPrivate::paint(GraphicsContext* context, const IntRect& rect)
     cairo_restore(cr);
 }
 
+static HashSet<String> mimeTypeCache()
+{
+    if (!gstInitialized) {
+        gst_init(0, NULL);
+        gstInitialized = true;
+    }
+
+    static HashSet<String> cache;
+    static bool typeListInitialized = false;
+
+    if (!typeListInitialized) {
+        // These subtypes are already beeing supported by WebKit itself
+        HashSet<String> ignoredApplicationSubtypes;
+        ignoredApplicationSubtypes.add(String("javascript"));
+        ignoredApplicationSubtypes.add(String("ecmascript"));
+        ignoredApplicationSubtypes.add(String("x-javascript"));
+        ignoredApplicationSubtypes.add(String("xml"));
+        ignoredApplicationSubtypes.add(String("xhtml+xml"));
+        ignoredApplicationSubtypes.add(String("rss+xml"));
+        ignoredApplicationSubtypes.add(String("atom+xml"));
+        ignoredApplicationSubtypes.add(String("x-ftp-directory"));
+        ignoredApplicationSubtypes.add(String("x-java-applet"));
+        ignoredApplicationSubtypes.add(String("x-java-bean"));
+        ignoredApplicationSubtypes.add(String("x-java-vm"));
+        ignoredApplicationSubtypes.add(String("x-shockwave-flash"));
+
+        GList* factories = gst_type_find_factory_get_list();
+        for (GList* iterator = factories; iterator; iterator = iterator->next) {
+            GstTypeFindFactory* factory = GST_TYPE_FIND_FACTORY(iterator->data);
+            GstCaps* caps = gst_type_find_factory_get_caps(factory);
+
+            // Splitting the capability by comma and taking the first part
+            // as capability can be something like "audio/x-wavpack, framed=(boolean)false"
+            GOwnPtr<gchar> capabilityString(gst_caps_to_string(caps));
+            gchar** capability = g_strsplit(capabilityString.get(), ",", 2);
+            gchar** mimetype = g_strsplit(capability[0], "/", 2);
+
+            // GStreamer plugins can be capable of supporting types which WebKit supports
+            // by default. In that case, we should not consider these types supportable by GStreamer.
+            // Examples of what GStreamer can support but should not be added:
+            // text/plain, text/html, image/jpeg, application/xml
+            if (g_str_equal(mimetype[0], "audio") ||
+                    g_str_equal(mimetype[0], "video") ||
+                    (g_str_equal(mimetype[0], "application") &&
+                        !ignoredApplicationSubtypes.contains(String(mimetype[1])))) {
+                cache.add(String(capability[0]));
+            }
+
+            g_strfreev(capability);
+            g_strfreev(mimetype);
+        }
+
+        gst_plugin_feature_list_free(factories);
+        typeListInitialized = true;
+    }
+
+    return cache;
+}
+
 void MediaPlayerPrivate::getSupportedTypes(HashSet<String>& types)
 {
-    // FIXME: query the engine to see what types are supported
-    notImplemented();
-    types.add(String("video/x-theora+ogg"));
+    types = mimeTypeCache();
 }
 
 MediaPlayer::SupportsType MediaPlayerPrivate::supportsType(const String& type, const String& codecs)
 {
-    // FIXME: query the engine to see what types are supported
-    notImplemented();
-    return type == "video/x-theora+ogg" ? (codecs.isEmpty() ? MediaPlayer::MayBeSupported : MediaPlayer::IsSupported) : MediaPlayer::IsNotSupported;
+    if (type.isNull() || type.isEmpty())
+        return MediaPlayer::IsNotSupported;
+
+    if (mimeTypeCache().contains(type))
+        return !codecs.isEmpty() ? MediaPlayer::MayBeSupported : MediaPlayer::IsSupported;
+    return MediaPlayer::IsNotSupported;
+}
+
+bool MediaPlayerPrivate::hasSingleSecurityOrigin() const
+{
+    return true;
+}
+
+bool MediaPlayerPrivate::supportsFullscreen() const
+{
+    return true;
 }
 
 void MediaPlayerPrivate::createGSTPlayBin(String url)
 {
     ASSERT(!m_playBin);
-    m_playBin = gst_element_factory_make("playbin", "play");
+    m_playBin = gst_element_factory_make("playbin2", "play");
 
     GstBus* bus = gst_pipeline_get_bus(GST_PIPELINE(m_playBin));
     gst_bus_add_signal_watch(bus);
@@ -632,10 +743,12 @@ void MediaPlayerPrivate::createGSTPlayBin(String url)
 
     g_object_set(G_OBJECT(m_playBin), "uri", url.utf8().data(), NULL);
 
-    GstElement* audioSink = gst_element_factory_make("gconfaudiosink", NULL);
     m_videoSink = webkit_video_sink_new(m_surface);
 
-    g_object_set(m_playBin, "audio-sink", audioSink, NULL);
+    // This ref is to protect the sink from being destroyed before we stop the idle it
+    // creates internally. See the comment in ~MediaPlayerPrivate.
+    g_object_ref(m_videoSink);
+
     g_object_set(m_playBin, "video-sink", m_videoSink, NULL);
 
     g_signal_connect(m_videoSink, "repaint-requested", G_CALLBACK(mediaPlayerPrivateRepaintCallback), this);
@@ -646,4 +759,3 @@ void MediaPlayerPrivate::createGSTPlayBin(String url)
 }
 
 #endif
-
