@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2006, 2007 Apple Inc. All rights reserved.
+ * Copyright (C) 2007-2009 Torch Mobile, Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -47,6 +48,9 @@
 #if PLATFORM(QT)
 #include "qt/TextCodecQt.h"
 #endif
+#if PLATFORM(WINCE) && !PLATFORM(QT)
+#include "TextCodecWince.h"
+#endif
 
 using namespace WTF;
 
@@ -189,6 +193,11 @@ static void buildBaseTextCodecMaps()
     TextCodecICU::registerBaseEncodingNames(addToTextEncodingNameMap);
     TextCodecICU::registerBaseCodecs(addToTextCodecMap);
 #endif
+
+#if PLATFORM(WINCE) && !PLATFORM(QT)
+    TextCodecWince::registerBaseEncodingNames(addToTextEncodingNameMap);
+    TextCodecWince::registerBaseCodecs(addToTextCodecMap);
+#endif
 }
 
 static void extendTextCodecMaps()
@@ -207,6 +216,11 @@ static void extendTextCodecMaps()
     TextCodecMac::registerEncodingNames(addToTextEncodingNameMap);
     TextCodecMac::registerCodecs(addToTextCodecMap);
 #endif
+
+#if PLATFORM(WINCE) && !PLATFORM(QT)
+    TextCodecWince::registerExtendedEncodingNames(addToTextEncodingNameMap);
+    TextCodecWince::registerExtendedCodecs(addToTextCodecMap);
+#endif
 }
 
 PassOwnPtr<TextCodec> newTextCodec(const TextEncoding& encoding)
