@@ -1,9 +1,7 @@
 /*
- * This file is part of the KDE project.
- *
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 2000 Simon Hausmann <hausmann@kde.org>
- * Copyright (C) 2006 Apple Computer, Inc.
+ * Copyright (C) 2006, 2009 Apple Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -31,17 +29,28 @@ namespace WebCore {
 
 class RenderPartObject : public RenderPart {
 public:
-    RenderPartObject(HTMLFrameOwnerElement*);
+    RenderPartObject(Element*);
     virtual ~RenderPartObject();
 
+    void updateWidget(bool onlyCreateNonNetscapePlugins);
+
+private:
     virtual const char* renderName() const { return "RenderPartObject"; }
 
     virtual void layout();
-    void updateWidget(bool onlyCreateNonNetscapePlugins);
 
     virtual void viewCleared();
 };
 
+inline RenderPartObject* toRenderPartObject(RenderObject* object)
+{
+    ASSERT(!object || !strcmp(object->renderName(), "RenderPartObject"));
+    return static_cast<RenderPartObject*>(object);
+}
+
+// This will catch anyone doing an unnecessary cast.
+void toRenderPartObject(const RenderPartObject*);
+
 } // namespace WebCore
 
 #endif // RenderPartObject_h
