@@ -1,8 +1,6 @@
 /*
     Copyright (C) 2007, 2008 Nikolas Zimmermann <zimmermann@kde.org>
 
-    This file is part of the KDE project
-
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
     License as published by the Free Software Foundation; either
@@ -27,9 +25,6 @@
 #include "SVGElement.h"
 #include "TreeShared.h"
 
-#include <wtf/RefPtr.h>
-#include <wtf/PassRefPtr.h>
-
 namespace WebCore {
 
     namespace Private { 
@@ -37,8 +32,6 @@ namespace WebCore {
         void addChildNodesToDeletionQueue(GenericNode*& head, GenericNode*& tail, GenericNodeContainer* container);
     };
 
-    class EventListener;
-    class Frame;
     class SVGUseElement;
     class SVGElementInstanceList;
 
@@ -46,7 +39,11 @@ namespace WebCore {
     class SVGElementInstance : public TreeShared<SVGElementInstance>,
                                public EventTarget {
     public:
-        SVGElementInstance(SVGUseElement*, SVGElement* originalElement);
+        static PassRefPtr<SVGElementInstance> create(SVGUseElement* useElement, PassRefPtr<SVGElement> originalElement)
+        {
+            return adoptRef(new SVGElementInstance(useElement, originalElement));
+        }
+
         virtual ~SVGElementInstance();
 
         bool needsUpdate() const { return m_needsUpdate; }
@@ -54,9 +51,6 @@ namespace WebCore {
 
         virtual ScriptExecutionContext* scriptExecutionContext() const;
 
-        virtual Node* toNode() { return shadowTreeElement(); }
-        virtual SVGElementInstance* toSVGElementInstance() { return this; }
-
         virtual void addEventListener(const AtomicString& eventType, PassRefPtr<EventListener>, bool useCapture);
         virtual void removeEventListener(const AtomicString& eventType, EventListener*, bool useCapture);
         virtual bool dispatchEvent(PassRefPtr<Event>, ExceptionCode&);
@@ -167,6 +161,11 @@ namespace WebCore {
     private:
         friend class SVGUseElement;
 
+        SVGElementInstance(SVGUseElement*, PassRefPtr<SVGElement> originalElement);
+
+        virtual Node* toNode() { return shadowTreeElement(); }
+        virtual SVGElementInstance* toSVGElementInstance() { return this; }
+
         void appendChild(PassRefPtr<SVGElementInstance> child);
         void setShadowTreeElement(SVGElement*);
         void forgetWrapper();
@@ -191,7 +190,6 @@ namespace WebCore {
         virtual void refEventTarget() { ref(); }
         virtual void derefEventTarget() { deref(); }
 
-    private:
         bool m_needsUpdate : 1;
 
         SVGUseElement* m_useElement;
