@@ -29,25 +29,29 @@
 #include "FloatPoint.h"
 #include "FloatRect.h"
 #include "Frame.h"
+#include "MappedAttribute.h"
 #include "Position.h"
 #include "RenderSVGText.h"
-#include "SelectionController.h"
 #include "SVGCharacterLayoutInfo.h"
-#include "SVGRootInlineBox.h"
-#include "SVGLength.h"
 #include "SVGInlineTextBox.h"
+#include "SVGLength.h"
 #include "SVGNames.h"
+#include "SVGRootInlineBox.h"
+#include "SelectionController.h"
 #include "XMLNames.h"
+#include <wtf/StdLibExtras.h>
 
 namespace WebCore {
 
+char SVGTextContentElementIdentifier[] = "SVGTextContentElement";
+
 SVGTextContentElement::SVGTextContentElement(const QualifiedName& tagName, Document* doc)
     : SVGStyledElement(tagName, doc)
     , SVGTests()
     , SVGLangSpace()
     , SVGExternalResourcesRequired()
-    , m_textLength(this, LengthModeOther)
-    , m_lengthAdjust(LENGTHADJUST_SPACING)
+    , m_textLength(this, SVGNames::textLengthAttr, LengthModeOther)
+    , m_lengthAdjust(this, SVGNames::lengthAdjustAttr, LENGTHADJUST_SPACING)
 {
 }
 
@@ -55,9 +59,6 @@ SVGTextContentElement::~SVGTextContentElement()
 {
 }
 
-ANIMATED_PROPERTY_DEFINITIONS(SVGTextContentElement, SVGLength, Length, length, TextLength, textLength, SVGNames::textLengthAttr, m_textLength)
-ANIMATED_PROPERTY_DEFINITIONS(SVGTextContentElement, int, Enumeration, enumeration, LengthAdjust, lengthAdjust, SVGNames::lengthAdjustAttr, m_lengthAdjust)
-
 static inline float cumulativeCharacterRangeLength(const Vector<SVGChar>::iterator& start, const Vector<SVGChar>::iterator& end, SVGInlineTextBox* textBox,
                                                    int startOffset, long startPosition, long length, bool isVerticalText, long& atCharacter)
 {
@@ -65,7 +66,7 @@ static inline float cumulativeCharacterRangeLength(const Vector<SVGChar>::iterat
         return 0.0f;
 
     float textLength = 0.0f;
-    RenderStyle* style = textBox->textObject()->style();
+    RenderStyle* style = textBox->textRenderer()->style();
 
     bool usesFullRange = (startPosition == -1 && length == -1);
 
@@ -125,10 +126,10 @@ struct SVGInlineTextBoxQueryWalker {
     {
     }
 
-    void chunkPortionCallback(SVGInlineTextBox* textBox, int startOffset, const AffineTransform& chunkCtm,
+    void chunkPortionCallback(SVGInlineTextBox* textBox, int startOffset, const TransformationMatrix&,
                               const Vector<SVGChar>::iterator& start, const Vector<SVGChar>::iterator& end)
     {
-        RenderStyle* style = textBox->textObject()->style();
+        RenderStyle* style = textBox->textRenderer()->style();
         bool isVerticalText = style->svgStyle()->writingMode() == WM_TBRL || style->svgStyle()->writingMode() == WM_TB;
 
         switch (m_mode) {
@@ -196,10 +197,13 @@ struct SVGInlineTextBoxQueryWalker {
 
                     int charsConsumed;
                     String glyphName;
+                    // calculateGlyph{Height,Width} will consume at least one character. This is the number of characters available
+                    // to them beyond that first one.
+                    int extraCharactersAvailable = end - it - 1;
                     if (isVerticalText)
-                        m_queryPointResult.move(it->x, it->y + textBox->calculateGlyphHeight(style, newOffset, end - it));
+                        m_queryPointResult.move(it->x, it->y + textBox->calculateGlyphHeight(style, newOffset, extraCharactersAvailable));
                     else
-                        m_queryPointResult.move(it->x + textBox->calculateGlyphWidth(style, newOffset, end - it, charsConsumed, glyphName), it->y);
+                        m_queryPointResult.move(it->x + textBox->calculateGlyphWidth(style, newOffset, extraCharactersAvailable, charsConsumed, glyphName), it->y);
 
                     m_stopProcessing = true;
                     return;
@@ -306,7 +310,7 @@ static Vector<SVGInlineTextBox*> findInlineTextBoxInTextChunks(const SVGTextCont
         for (; boxIt != boxEnd; ++boxIt) {
             SVGInlineTextBox* textBox = static_cast<SVGInlineTextBox*>(boxIt->box);
 
-            Node* textElement = textBox->textObject()->parent()->element();
+            Node* textElement = textBox->textRenderer()->parent()->node();
             ASSERT(textElement);
 
             if (textElement == element || textElement->parent() == element)
@@ -324,7 +328,7 @@ static inline SVGRootInlineBox* rootInlineBoxForTextContentElement(const SVGText
     if (!object || !object->isSVGText() || object->isText())
         return 0;
 
-    RenderSVGText* svgText = static_cast<RenderSVGText*>(object);
+    RenderBlock* svgText = toRenderBlock(object);
 
     // Find root inline box
     SVGRootInlineBox* rootBox = static_cast<SVGRootInlineBox*>(svgText->firstRootBox());
@@ -367,7 +371,7 @@ static inline SVGInlineTextBoxQueryWalker executeTextQuery(const SVGTextContentE
     return walkerCallback;
 }
 
-long SVGTextContentElement::getNumberOfChars() const
+unsigned SVGTextContentElement::getNumberOfChars() const
 {
     document()->updateLayoutIgnorePendingStylesheets();
 
@@ -381,23 +385,12 @@ float SVGTextContentElement::getComputedTextLength() const
     return executeTextQuery(this, SVGInlineTextBoxQueryWalker::TextLength).floatResult();
 }
 
-float SVGTextContentElement::getSubStringLength(long charnum, long nchars, ExceptionCode& ec) const
+float SVGTextContentElement::getSubStringLength(unsigned charnum, unsigned nchars, ExceptionCode& ec) const
 {
     document()->updateLayoutIgnorePendingStylesheets();
 
-    // Differences to SVG 1.1 spec, as the spec is clearly wrong. TODO: Raise SVG WG issue!
-    // #1: We accept a 'long nchars' parameter instead of 'unsigned long nchars' to be able
-    //     to catch cases where someone called us with a negative 'nchars' value - in those
-    //     cases we'll just throw a 'INDEX_SIZE_ERR' (acid3 implicitly agrees with us)
-    //
-    // #2: We only throw if 'charnum + nchars' is greater than the number of characters, not
-    //     if it's equal, as this really doesn't make any sense (no way to measure the last character!)
-    //
-    // #3: If 'charnum' is greater than or equal to 'numberOfChars', we're throwing an exception here
-    //     as the result is undefined for every other value of 'nchars' than '0'.
-
-    long numberOfChars = getNumberOfChars();
-    if (charnum < 0 || nchars < 0 || numberOfChars <= charnum || charnum + nchars > numberOfChars) {
+    unsigned numberOfChars = getNumberOfChars();
+    if (charnum >= numberOfChars) {
         ec = INDEX_SIZE_ERR;
         return 0.0f;
     }
@@ -405,11 +398,11 @@ float SVGTextContentElement::getSubStringLength(long charnum, long nchars, Excep
     return executeTextQuery(this, SVGInlineTextBoxQueryWalker::SubStringLength, charnum, nchars).floatResult();
 }
 
-FloatPoint SVGTextContentElement::getStartPositionOfChar(long charnum, ExceptionCode& ec) const
+FloatPoint SVGTextContentElement::getStartPositionOfChar(unsigned charnum, ExceptionCode& ec) const
 {
     document()->updateLayoutIgnorePendingStylesheets();
 
-    if (charnum < 0 || charnum > getNumberOfChars()) {
+    if (charnum > getNumberOfChars()) {
         ec = INDEX_SIZE_ERR;
         return FloatPoint();
     }
@@ -417,11 +410,11 @@ FloatPoint SVGTextContentElement::getStartPositionOfChar(long charnum, Exception
     return executeTextQuery(this, SVGInlineTextBoxQueryWalker::StartPosition, charnum).pointResult();
 }
 
-FloatPoint SVGTextContentElement::getEndPositionOfChar(long charnum, ExceptionCode& ec) const
+FloatPoint SVGTextContentElement::getEndPositionOfChar(unsigned charnum, ExceptionCode& ec) const
 {
     document()->updateLayoutIgnorePendingStylesheets();
 
-    if (charnum < 0 || charnum > getNumberOfChars()) {
+    if (charnum > getNumberOfChars()) {
         ec = INDEX_SIZE_ERR;
         return FloatPoint();
     }
@@ -429,11 +422,11 @@ FloatPoint SVGTextContentElement::getEndPositionOfChar(long charnum, ExceptionCo
     return executeTextQuery(this, SVGInlineTextBoxQueryWalker::EndPosition, charnum).pointResult();
 }
 
-FloatRect SVGTextContentElement::getExtentOfChar(long charnum, ExceptionCode& ec) const
+FloatRect SVGTextContentElement::getExtentOfChar(unsigned charnum, ExceptionCode& ec) const
 {
     document()->updateLayoutIgnorePendingStylesheets();
 
-    if (charnum < 0 || charnum > getNumberOfChars()) {
+    if (charnum > getNumberOfChars()) {
         ec = INDEX_SIZE_ERR;
         return FloatRect();
     }
@@ -441,11 +434,11 @@ FloatRect SVGTextContentElement::getExtentOfChar(long charnum, ExceptionCode& ec
     return executeTextQuery(this, SVGInlineTextBoxQueryWalker::Extent, charnum).rectResult();
 }
 
-float SVGTextContentElement::getRotationOfChar(long charnum, ExceptionCode& ec) const
+float SVGTextContentElement::getRotationOfChar(unsigned charnum, ExceptionCode& ec) const
 {
     document()->updateLayoutIgnorePendingStylesheets();
 
-    if (charnum < 0 || charnum > getNumberOfChars()) {
+    if (charnum > getNumberOfChars()) {
         ec = INDEX_SIZE_ERR;
         return 0.0f;
     }
@@ -453,17 +446,17 @@ float SVGTextContentElement::getRotationOfChar(long charnum, ExceptionCode& ec)
     return executeTextQuery(this, SVGInlineTextBoxQueryWalker::Rotation, charnum).floatResult();
 }
 
-long SVGTextContentElement::getCharNumAtPosition(const FloatPoint& point) const
+int SVGTextContentElement::getCharNumAtPosition(const FloatPoint& point) const
 {
     document()->updateLayoutIgnorePendingStylesheets();
 
     return executeTextQuery(this, SVGInlineTextBoxQueryWalker::CharacterNumberAtPosition, 0.0f, 0.0f, point).longResult();
 }
 
-void SVGTextContentElement::selectSubString(long charnum, long nchars, ExceptionCode& ec) const
+void SVGTextContentElement::selectSubString(unsigned charnum, unsigned nchars, ExceptionCode& ec) const
 {
-    long numberOfChars = getNumberOfChars();
-    if (charnum < 0 || nchars < 0 || charnum > numberOfChars) {
+    unsigned numberOfChars = getNumberOfChars();
+    if (charnum >= numberOfChars) {
         ec = INDEX_SIZE_ERR;
         return;
     }
@@ -474,21 +467,21 @@ void SVGTextContentElement::selectSubString(long charnum, long nchars, Exception
     ASSERT(document());
     ASSERT(document()->frame());
 
-    SelectionController* controller = document()->frame()->selectionController();
+    SelectionController* controller = document()->frame()->selection();
     if (!controller)
         return;
 
     // Find selection start
     VisiblePosition start(const_cast<SVGTextContentElement*>(this), 0, SEL_DEFAULT_AFFINITY);
-    for (long i = 0; i < charnum; ++i)
+    for (unsigned i = 0; i < charnum; ++i)
         start = start.next();
 
     // Find selection end
     VisiblePosition end(start);
-    for (long i = 0; i < nchars; ++i)
+    for (unsigned i = 0; i < nchars; ++i)
         end = end.next();
 
-    controller->setSelection(Selection(start, end));
+    controller->setSelection(VisibleSelection(start, end));
 }
 
 void SVGTextContentElement::parseMappedAttribute(MappedAttribute* attr)
@@ -499,15 +492,15 @@ void SVGTextContentElement::parseMappedAttribute(MappedAttribute* attr)
         else if (attr->value() == "spacingAndGlyphs")
             setLengthAdjustBaseValue(LENGTHADJUST_SPACINGANDGLYPHS);
     } else if (attr->name() == SVGNames::textLengthAttr) {
-        setTextLengthBaseValue(SVGLength(this, LengthModeOther, attr->value()));
-        if (textLength().value() < 0.0)
+        setTextLengthBaseValue(SVGLength(LengthModeOther, attr->value()));
+        if (textLengthBaseValue().value(this) < 0.0)
             document()->accessSVGExtensions()->reportError("A negative value for text attribute <textLength> is not allowed");
     } else {
         if (SVGTests::parseMappedAttribute(attr))
             return;
         if (SVGLangSpace::parseMappedAttribute(attr)) {
             if (attr->name().matches(XMLNames::spaceAttr)) {
-                static const AtomicString preserveString("preserve");
+                DEFINE_STATIC_LOCAL(const AtomicString, preserveString, ("preserve"));
 
                 if (attr->value() == preserveString)
                     addCSSProperty(attr, CSSPropertyWhiteSpace, CSSValuePre);
