@@ -1,6 +1,7 @@
 /*
     Copyright (C) 2004, 2005, 2006, 2007, 2008 Nikolas Zimmermann <zimmermann@kde.org>
                   2004, 2005, 2006, 2007 Rob Buis <buis@kde.org>
+    Copyright (C) Research In Motion Limited 2009, All rights reserved. Charles Wei <charles.wei@torchmobile.com.cn>
 
     This file is part of the KDE project
 
@@ -22,12 +23,6 @@
 
 #include "config.h"
 
-// Dump SVGElementInstance object tree - useful to debug instanceRoot problems
-// #define DUMP_INSTANCE_TREE
-
-// Dump the deep-expanded shadow tree (where the renderes are built from)
-// #define DUMP_SHADOW_TREE
-
 #if ENABLE(SVG)
 #include "SVGUseElement.h"
 
@@ -35,20 +30,28 @@
 #include "CString.h"
 #include "Document.h"
 #include "Event.h"
+#include "EventListener.h"
 #include "HTMLNames.h"
+#include "MappedAttribute.h"
+#include "NodeRenderStyle.h"
+#include "RegisteredEventListener.h"
 #include "RenderSVGTransformableContainer.h"
 #include "SVGElementInstance.h"
 #include "SVGElementInstanceList.h"
 #include "SVGGElement.h"
 #include "SVGLength.h"
-#include "SVGNames.h"
 #include "SVGPreserveAspectRatio.h"
 #include "SVGSMILElement.h"
 #include "SVGSVGElement.h"
 #include "SVGSymbolElement.h"
 #include "XLinkNames.h"
 #include "XMLSerializer.h"
-#include <wtf/OwnPtr.h>
+
+// Dump SVGElementInstance object tree - useful to debug instanceRoot problems
+// #define DUMP_INSTANCE_TREE
+
+// Dump the deep-expanded shadow tree (where the renderers are built from)
+// #define DUMP_SHADOW_TREE
 
 namespace WebCore {
 
@@ -58,10 +61,10 @@ SVGUseElement::SVGUseElement(const QualifiedName& tagName, Document* doc)
     , SVGLangSpace()
     , SVGExternalResourcesRequired()
     , SVGURIReference()
-    , m_x(this, LengthModeWidth)
-    , m_y(this, LengthModeHeight)
-    , m_width(this, LengthModeWidth)
-    , m_height(this, LengthModeHeight)
+    , m_x(this, SVGNames::xAttr, LengthModeWidth)
+    , m_y(this, SVGNames::yAttr, LengthModeHeight)
+    , m_width(this, SVGNames::widthAttr, LengthModeWidth)
+    , m_height(this, SVGNames::heightAttr, LengthModeHeight)
 {
 }
 
@@ -69,11 +72,6 @@ SVGUseElement::~SVGUseElement()
 {
 }
 
-ANIMATED_PROPERTY_DEFINITIONS(SVGUseElement, SVGLength, Length, length, X, x, SVGNames::xAttr, m_x)
-ANIMATED_PROPERTY_DEFINITIONS(SVGUseElement, SVGLength, Length, length, Y, y, SVGNames::yAttr, m_y)
-ANIMATED_PROPERTY_DEFINITIONS(SVGUseElement, SVGLength, Length, length, Width, width, SVGNames::widthAttr, m_width)
-ANIMATED_PROPERTY_DEFINITIONS(SVGUseElement, SVGLength, Length, length, Height, height, SVGNames::heightAttr, m_height)
-
 SVGElementInstance* SVGUseElement::instanceRoot() const
 {
     return m_targetElementInstance.get();
@@ -88,16 +86,16 @@ SVGElementInstance* SVGUseElement::animatedInstanceRoot() const
 void SVGUseElement::parseMappedAttribute(MappedAttribute* attr)
 {
     if (attr->name() == SVGNames::xAttr)
-        setXBaseValue(SVGLength(this, LengthModeWidth, attr->value()));
+        setXBaseValue(SVGLength(LengthModeWidth, attr->value()));
     else if (attr->name() == SVGNames::yAttr)
-        setYBaseValue(SVGLength(this, LengthModeHeight, attr->value()));
+        setYBaseValue(SVGLength(LengthModeHeight, attr->value()));
     else if (attr->name() == SVGNames::widthAttr) {
-        setWidthBaseValue(SVGLength(this, LengthModeWidth, attr->value()));
-        if (width().value() < 0.0)
+        setWidthBaseValue(SVGLength(LengthModeWidth, attr->value()));
+        if (widthBaseValue().value(this) < 0.0)
             document()->accessSVGExtensions()->reportError("A negative value for use attribute <width> is not allowed");
     } else if (attr->name() == SVGNames::heightAttr) {
-        setHeightBaseValue(SVGLength(this, LengthModeHeight, attr->value()));
-        if (height().value() < 0.0)
+        setHeightBaseValue(SVGLength(LengthModeHeight, attr->value()));
+        if (heightBaseValue().value(this) < 0.0)
             document()->accessSVGExtensions()->reportError("A negative value for use attribute <height> is not allowed");
     } else {
         if (SVGTests::parseMappedAttribute(attr))
@@ -139,12 +137,10 @@ void SVGUseElement::svgAttributeChanged(const QualifiedName& attrName)
         SVGExternalResourcesRequired::isKnownAttribute(attrName) ||
         SVGURIReference::isKnownAttribute(attrName) ||
         SVGStyledTransformableElement::isKnownAttribute(attrName)) {
-        // TODO: Now that we're aware of the attribute name, we can finally optimize
-        // updating <use> attributes - to not reclone every time.
         buildPendingResource();
 
         if (m_shadowTreeRootElement)
-            m_shadowTreeRootElement->setChanged();
+            m_shadowTreeRootElement->setNeedsStyleRecalc();
     }
 }
 
@@ -158,11 +154,33 @@ void SVGUseElement::childrenChanged(bool changedByParser, Node* beforeChange, No
     buildPendingResource();
 
     if (m_shadowTreeRootElement)
-        m_shadowTreeRootElement->setChanged();
+        m_shadowTreeRootElement->setNeedsStyleRecalc();
+}
+ 
+static bool shadowTreeContainsChangedNodes(SVGElementInstance* target)
+{
+    if (!target)      // when use is referencing an non-existing element, there will be no Instance tree built
+        return false;
+
+    if (target->needsUpdate())
+        return true;
+
+    for (SVGElementInstance* instance = target->firstChild(); instance; instance = instance->nextSibling())
+        if (shadowTreeContainsChangedNodes(instance))
+            return true;
+
+    return false;
 }
 
 void SVGUseElement::recalcStyle(StyleChange change)
 {
+    if (attached() && needsStyleRecalc() && shadowTreeContainsChangedNodes(m_targetElementInstance.get())) {
+        buildPendingResource();
+
+        if (m_shadowTreeRootElement)
+            m_shadowTreeRootElement->setNeedsStyleRecalc();
+    }
+
     SVGStyledElement::recalcStyle(change);
 
     // The shadow tree root element is NOT a direct child element of us.
@@ -173,22 +191,19 @@ void SVGUseElement::recalcStyle(StyleChange change)
     // Mimic Element::recalcStyle(). The main difference is that we don't call attach() on the
     // shadow tree root element, but call attachShadowTree() here. Calling attach() will crash
     // as the shadow tree root element has no (direct) parent node. Yes, shadow trees are tricky.
-    if (change >= Inherit || m_shadowTreeRootElement->changed()) {
-        RenderStyle* newStyle = document()->styleSelector()->styleForElement(m_shadowTreeRootElement.get());
-        StyleChange ch = m_shadowTreeRootElement->diff(m_shadowTreeRootElement->renderStyle(), newStyle);
+    if (change >= Inherit || m_shadowTreeRootElement->needsStyleRecalc()) {
+        RefPtr<RenderStyle> newStyle = document()->styleSelector()->styleForElement(m_shadowTreeRootElement.get());
+        StyleChange ch = Node::diff(m_shadowTreeRootElement->renderStyle(), newStyle.get());
         if (ch == Detach) {
             ASSERT(m_shadowTreeRootElement->attached());
             m_shadowTreeRootElement->detach();
             attachShadowTree();
 
             // attach recalulates the style for all children. No need to do it twice.
-            m_shadowTreeRootElement->setChanged(NoStyleChange);
-            m_shadowTreeRootElement->setHasChangedChild(false);
-            newStyle->deref(document()->renderArena());
+            m_shadowTreeRootElement->setNeedsStyleRecalc(NoStyleChange);
+            m_shadowTreeRootElement->setChildNeedsStyleRecalc(false);
             return;
         }
-
-        newStyle->deref(document()->renderArena());
     }
 
     // Only change==Detach needs special treatment, for anything else recalcStyle() works.
@@ -209,15 +224,31 @@ void dumpInstanceTree(unsigned int& depth, String& text, SVGElementInstance* tar
     for (unsigned int i = 0; i < depth; ++i)
         text += "  ";
 
-    text += String::format("SVGElementInstance (parentNode=%s, firstChild=%s, correspondingElement=%s, id=%s)\n",
-                           parentNodeName.latin1().data(), firstChildNodeName.latin1().data(), elementNodeName.latin1().data(), elementId.latin1().data());
- 
-    depth++;
+    text += String::format("SVGElementInstance this=%p, (parentNode=%s, firstChild=%s, correspondingElement=%s (%p), shadowTreeElement=%p, id=%s)\n",
+                           targetInstance, parentNodeName.latin1().data(), firstChildNodeName.latin1().data(), elementNodeName.latin1().data(),
+                           element, targetInstance->shadowTreeElement(), elementId.latin1().data());
+
+    for (unsigned int i = 0; i < depth; ++i)
+        text += "  ";
+
+    HashSet<SVGElementInstance*> elementInstances = element->instancesForElement();
+    text += String::format("Corresponding element is associated with %i instance(s):\n", elementInstances.size());
+
+    HashSet<SVGElementInstance*>::iterator end = elementInstances.end();
+    for (HashSet<SVGElementInstance*>::iterator it = elementInstances.begin(); it != end; ++it) {
+        for (unsigned int i = 0; i < depth; ++i)
+            text += "  ";
+
+        text += String::format(" -> SVGElementInstance this=%p, (refCount: %i, shadowTreeElement in document? %i)\n",
+                               *it, (*it)->refCount(), (*it)->shadowTreeElement()->inDocument());
+    }
+
+    ++depth;
 
     for (SVGElementInstance* instance = targetInstance->firstChild(); instance; instance = instance->nextSibling())
         dumpInstanceTree(depth, text, instance);
 
-    depth--;
+    --depth;
 }
 #endif
 
@@ -275,10 +306,14 @@ void SVGUseElement::buildPendingResource()
     if (targetElement && targetElement->isSVGElement())
         target = static_cast<SVGElement*>(targetElement);
 
+    if (m_targetElementInstance) {
+        m_targetElementInstance->forgetWrapper();
+        m_targetElementInstance = 0;
+    }
+
     // Do not allow self-referencing.
     // 'target' may be null, if it's a non SVG namespaced element.
     if (!target || target == this) {
-        m_targetElementInstance = 0;
         m_shadowTreeRootElement = 0;
         return;
     }
@@ -293,7 +328,7 @@ void SVGUseElement::buildPendingResource()
     // Spec: If the 'use' element references a simple graphics element such as a 'rect', then there is only a
     // single SVGElementInstance object, and the correspondingElement attribute on this SVGElementInstance object
     // is the SVGRectElement that corresponds to the referenced 'rect' element.
-    m_targetElementInstance = new SVGElementInstance(this, target);
+    m_targetElementInstance = SVGElementInstance::create(this, target);
 
     // Eventually enter recursion to build SVGElementInstance objects for the sub-tree children
     bool foundProblem = false;
@@ -319,8 +354,8 @@ void SVGUseElement::buildPendingResource()
     // Spec: An additional transformation translate(x,y) is appended to the end
     // (i.e., right-side) of the transform attribute on the generated 'g', where x
     // and y represent the values of the x and y attributes on the 'use' element. 
-    if (x().value() != 0.0 || y().value() != 0.0) {
-        String transformString = String::format("translate(%f, %f)", x().value(), y().value());
+    if (x().value(this) != 0.0 || y().value(this) != 0.0) {
+        String transformString = String::format("translate(%f, %f)", x().value(this), y().value(this));
         m_shadowTreeRootElement->setAttribute(SVGNames::transformAttr, transformString);
     }
 
@@ -364,6 +399,9 @@ void SVGUseElement::buildPendingResource()
     fprintf(stderr, "Dumping <use> shadow tree markup:\n%s\n", markup.latin1().data());
 #endif
 
+    // Transfer event listeners assigned to the referenced element to our shadow tree elements.
+    transferEventListenersToShadowTree(m_targetElementInstance.get());
+
     // The DOM side is setup properly. Now we have to attach the root shadow
     // tree element manually - using attach() won't work for "shadow nodes".
     attachShadowTree();
@@ -440,28 +478,26 @@ void SVGUseElement::buildInstanceTree(SVGElement* target, SVGElementInstance* ta
             continue;
 
         // Create SVGElementInstance object, for both container/non-container nodes.
-        SVGElementInstance* instancePtr = new SVGElementInstance(this, element);
-
-        RefPtr<SVGElementInstance> instance = instancePtr;
-        targetInstance->appendChild(instance.release());
+        RefPtr<SVGElementInstance> instancePtr = SVGElementInstance::create(this, element);
+        targetInstance->appendChild(instancePtr.get());
 
         // Enter recursion, appending new instance tree nodes to the "instance" object.
         if (element->hasChildNodes())
-            buildInstanceTree(element, instancePtr, foundProblem);
+            buildInstanceTree(element, instancePtr.get(), foundProblem);
 
         // Spec: If the referenced object is itself a 'use', or if there are 'use' subelements within the referenced
         // object, the instance tree will contain recursive expansion of the indirect references to form a complete tree.
         if (element->hasTagName(SVGNames::useTag))
-            handleDeepUseReferencing(element, instancePtr, foundProblem);
+            handleDeepUseReferencing(static_cast<SVGUseElement*>(element), instancePtr.get(), foundProblem);
     }
 
     // Spec: If the referenced object is itself a 'use', or if there are 'use' subelements within the referenced
     // object, the instance tree will contain recursive expansion of the indirect references to form a complete tree.
     if (target->hasTagName(SVGNames::useTag))
-        handleDeepUseReferencing(target, targetInstance, foundProblem);
+        handleDeepUseReferencing(static_cast<SVGUseElement*>(target), targetInstance, foundProblem);
 }
 
-void SVGUseElement::handleDeepUseReferencing(SVGElement* use, SVGElementInstance* targetInstance, bool& foundProblem)
+void SVGUseElement::handleDeepUseReferencing(SVGUseElement* use, SVGElementInstance* targetInstance, bool& foundProblem)
 {
     String id = SVGURIReference::getTarget(use->href());
     Element* targetElement = document()->getElementById(id); 
@@ -492,17 +528,17 @@ void SVGUseElement::handleDeepUseReferencing(SVGElement* use, SVGElementInstance
     }
 
     // Create an instance object, even if we're dealing with a cycle
-    SVGElementInstance* newInstance = new SVGElementInstance(this, target);
+    RefPtr<SVGElementInstance> newInstance = SVGElementInstance::create(this, target);
     targetInstance->appendChild(newInstance);
 
     // Eventually enter recursion to build SVGElementInstance objects for the sub-tree children
-    buildInstanceTree(target, newInstance, foundProblem);
+    buildInstanceTree(target, newInstance.get(), foundProblem);
 }
 
 void SVGUseElement::alterShadowTreeForSVGTag(SVGElement* target)
 {
-    String widthString = String::number(width().value());
-    String heightString = String::number(height().value());
+    String widthString = String::number(width().value(this));
+    String heightString = String::number(height().value(this));
 
     if (hasAttribute(SVGNames::widthAttr))
         target->setAttribute(SVGNames::widthAttr, widthString);
@@ -533,10 +569,10 @@ void SVGUseElement::buildShadowTree(SVGElement* target, SVGElementInstance* targ
     if (isDisallowedElement(target))
         return;
 
-    RefPtr<Node> newChild = targetInstance->correspondingElement()->cloneNode(true);
+    RefPtr<Element> newChild = targetInstance->correspondingElement()->cloneElementWithChildren();
 
     // We don't walk the target tree element-by-element, and clone each element,
-    // but instead use cloneNode(deep=true). This is an optimization for the common
+    // but instead use cloneElementWithChildren(). This is an optimization for the common
     // case where <use> doesn't contain disallowed elements (ie. <foreignObject>).
     // Though if there are disallowed elements in the subtree, we have to remove them.
     // For instance: <use> on <g> containing <foreignObject> (indirect case).
@@ -588,12 +624,12 @@ void SVGUseElement::expandUseElementsInShadowTree(Node* element)
             // Spec: An additional transformation translate(x,y) is appended to the end
             // (i.e., right-side) of the transform attribute on the generated 'g', where x
             // and y represent the values of the x and y attributes on the 'use' element.
-            if (use->x().value() != 0.0 || use->y().value() != 0.0) {
+            if (use->x().value(this) != 0.0 || use->y().value(this) != 0.0) {
                 if (!cloneParent->hasAttribute(SVGNames::transformAttr)) {
-                    String transformString = String::format("translate(%f, %f)", use->x().value(), use->y().value());
+                    String transformString = String::format("translate(%f, %f)", use->x().value(this), use->y().value(this));
                     cloneParent->setAttribute(SVGNames::transformAttr, transformString);
                 } else {
-                    String transformString = String::format(" translate(%f, %f)", use->x().value(), use->y().value());
+                    String transformString = String::format(" translate(%f, %f)", use->x().value(this), use->y().value(this));
                     const AtomicString& transformAttribute = cloneParent->getAttribute(SVGNames::transformAttr);
                     cloneParent->setAttribute(SVGNames::transformAttr, transformAttribute + transformString); 
                 }
@@ -612,10 +648,10 @@ void SVGUseElement::expandUseElementsInShadowTree(Node* element)
                 return;
             }
 
-            RefPtr<Node> newChild = target->cloneNode(true);
+            RefPtr<Element> newChild = target->cloneElementWithChildren();
 
             // We don't walk the target tree element-by-element, and clone each element,
-            // but instead use cloneNode(deep=true). This is an optimization for the common
+            // but instead use cloneElementWithChildren(). This is an optimization for the common
             // case where <use> doesn't contain disallowed elements (ie. <foreignObject>).
             // Though if there are disallowed elements in the subtree, we have to remove them.
             // For instance: <use> on <g> containing <foreignObject> (indirect case).
@@ -664,8 +700,8 @@ void SVGUseElement::expandSymbolElementsInShadowTree(Node* element)
         svgElement->attributes()->setAttributes(*element->attributes());
 
         // Explicitly re-set width/height values
-        String widthString = String::number(width().value());
-        String heightString = String::number(height().value()); 
+        String widthString = String::number(width().value(this));
+        String heightString = String::number(height().value(this)); 
 
         svgElement->setAttribute(SVGNames::widthAttr, hasAttribute(SVGNames::widthAttr) ? widthString : "100%");
         svgElement->setAttribute(SVGNames::heightAttr, hasAttribute(SVGNames::heightAttr) ? heightString : "100%");
@@ -710,24 +746,50 @@ void SVGUseElement::attachShadowTree()
 
     // Inspired by RenderTextControl::createSubtreeIfNeeded(). 
     if (renderer()->canHaveChildren() && childShouldCreateRenderer(m_shadowTreeRootElement.get())) {
-        RenderStyle* style = m_shadowTreeRootElement->styleForRenderer(renderer());
+        RefPtr<RenderStyle> style = m_shadowTreeRootElement->styleForRenderer();
 
-        if (m_shadowTreeRootElement->rendererIsNeeded(style)) {
-            m_shadowTreeRootElement->setRenderer(m_shadowTreeRootElement->createRenderer(document()->renderArena(), style));
+        if (m_shadowTreeRootElement->rendererIsNeeded(style.get())) {
+            m_shadowTreeRootElement->setRenderer(m_shadowTreeRootElement->createRenderer(document()->renderArena(), style.get()));
             if (RenderObject* shadowRenderer = m_shadowTreeRootElement->renderer()) {
-                shadowRenderer->setStyle(style);
+                shadowRenderer->setStyle(style.release());
                 renderer()->addChild(shadowRenderer, m_shadowTreeRootElement->nextRenderer());
                 m_shadowTreeRootElement->setAttached();
             }
         }
 
-        style->deref(document()->renderArena());
-
         // This will take care of attaching all shadow tree child nodes.
         for (Node* child = m_shadowTreeRootElement->firstChild(); child; child = child->nextSibling())
             child->attach();
     }
 }
+ 
+void SVGUseElement::transferEventListenersToShadowTree(SVGElementInstance* target)
+{
+    if (!target)
+        return;
+
+    SVGElement* originalElement = target->correspondingElement();
+    ASSERT(originalElement);
+
+    if (SVGElement* shadowTreeElement = target->shadowTreeElement()) {
+        const RegisteredEventListenerVector& listeners = originalElement->eventListeners();
+        size_t size = listeners.size();
+        for (size_t i = 0; i < size; ++i) {
+            const RegisteredEventListener& r = *listeners[i];
+            EventListener* listener = r.listener();
+            ASSERT(listener);
+
+            // Event listeners created from markup have already been transfered to the shadow tree during cloning!
+            if (listener->wasCreatedFromMarkup())
+                continue;
+
+            shadowTreeElement->addEventListener(r.eventType(), listener, r.useCapture());
+        }
+    }
+
+    for (SVGElementInstance* instance = target->firstChild(); instance; instance = instance->nextSibling())
+        transferEventListenersToShadowTree(instance);
+}
 
 void SVGUseElement::associateInstancesWithShadowTreeElements(Node* target, SVGElementInstance* targetInstance)
 {
@@ -778,7 +840,11 @@ SVGElementInstance* SVGUseElement::instanceForShadowTreeElement(Node* element, S
 {
     ASSERT(element);
     ASSERT(instance);
-    ASSERT(instance->shadowTreeElement());
+
+    // We're dispatching a mutation event during shadow tree construction
+    // this instance hasn't yet been associated to a shadowTree element.
+    if (!instance->shadowTreeElement())
+        return 0;
 
     if (element == instance->shadowTreeElement())
         return instance;
