@@ -32,6 +32,7 @@
 #include <JavaScriptCore/JSObjectRef.h>
 #include <wtf/RefCounted.h>
 #include <string>
+#include <vector>
 
 class LayoutTestController : public RefCounted<LayoutTestController> {
 public:
@@ -46,12 +47,14 @@ public:
     void clearPersistentUserStyleSheet();
     JSStringRef copyDecodedHostName(JSStringRef name);
     JSStringRef copyEncodedHostName(JSStringRef name);
+    void disableImageLoading();
     void dispatchPendingLoadRequests();
     void display();
     void execCommand(JSStringRef name, JSStringRef value);
     bool isCommandEnabled(JSStringRef name);
     void keepWebHistory();
     void notifyDone();
+    void overridePreference(JSStringRef key, JSStringRef value);
     JSStringRef pathToLocalResource(JSContextRef, JSStringRef url);
     void queueBackNavigation(int howFarBackward);
     void queueForwardNavigation(int howFarForward);
@@ -65,6 +68,8 @@ public:
     void setCacheModel(int);
     void setCustomPolicyDelegate(bool setDelegate, bool permissive);
     void setDatabaseQuota(unsigned long long quota);
+    void setMockGeolocationPosition(double latitude, double longitude, double accuracy);
+    void setMockGeolocationError(int code, JSStringRef message);
     void setIconDatabaseEnabled(bool iconDatabaseEnabled);
     void setJavaScriptProfilingEnabled(bool profilingEnabled);
     void setMainFrameIsFirstResponder(bool flag);
@@ -80,8 +85,12 @@ public:
     void setUserStyleSheetLocation(JSStringRef path);
     void waitForPolicyDelegate();
     size_t webHistoryItemCount();
+    unsigned workerThreadCount() const;
     int windowCount();
     
+    void grantDesktopNotificationPermission(JSStringRef origin);
+    bool checkDesktopNotificationPermission(JSStringRef origin);
+
     bool elementDoesAutoCompleteForElementWithId(JSStringRef id);
 
     bool dumpAsText() const { return m_dumpAsText; }
@@ -126,6 +135,9 @@ public:
     bool dumpResourceResponseMIMETypes() const { return m_dumpResourceResponseMIMETypes; }
     void setDumpResourceResponseMIMETypes(bool dumpResourceResponseMIMETypes) { m_dumpResourceResponseMIMETypes = dumpResourceResponseMIMETypes; }
 
+    bool dumpWillCacheResponse() const { return m_dumpWillCacheResponse; }
+    void setDumpWillCacheResponse(bool dumpWillCacheResponse) { m_dumpWillCacheResponse = dumpWillCacheResponse; }
+
     bool dumpFrameLoadCallbacks() const { return m_dumpFrameLoadCallbacks; }
     void setDumpFrameLoadCallbacks(bool dumpFrameLoadCallbacks) { m_dumpFrameLoadCallbacks = dumpFrameLoadCallbacks; }
 
@@ -152,6 +164,7 @@ public:
 
     bool waitToDump() const { return m_waitToDump; }
     void setWaitToDump(bool waitToDump);
+    void waitToDumpWatchdogTimerFired();
 
     bool willSendRequestReturnsNullOnRedirect() const { return m_willSendRequestReturnsNullOnRedirect; }
     void setWillSendRequestReturnsNullOnRedirect(bool returnsNull) { m_willSendRequestReturnsNullOnRedirect = returnsNull; }
@@ -159,6 +172,18 @@ public:
     bool windowIsKey() const { return m_windowIsKey; }
     void setWindowIsKey(bool windowIsKey);
 
+    bool alwaysAcceptCookies() const { return m_alwaysAcceptCookies; }
+    void setAlwaysAcceptCookies(bool alwaysAcceptCookies);
+    
+    bool handlesAuthenticationChallenges() const { return m_handlesAuthenticationChallenges; }
+    void setHandlesAuthenticationChallenges(bool handlesAuthenticationChallenges) { m_handlesAuthenticationChallenges = handlesAuthenticationChallenges; }
+    
+    const std::string& authenticationUsername() const { return m_authenticationUsername; }
+    void setAuthenticationUsername(std::string username) { m_authenticationUsername = username; }
+    
+    const std::string& authenticationPassword() const { return m_authenticationPassword; }
+    void setAuthenticationPassword(std::string password) { m_authenticationPassword = password; }
+
     bool globalFlag() const { return m_globalFlag; }
     void setGlobalFlag(bool globalFlag) { m_globalFlag = globalFlag; }
     
@@ -168,7 +193,16 @@ public:
     bool pauseAnimationAtTimeOnElementWithId(JSStringRef animationName, double time, JSStringRef elementId);
     bool pauseTransitionAtTimeOnElementWithId(JSStringRef propertyName, double time, JSStringRef elementId);
     unsigned numberOfActiveAnimations() const;
-    
+
+    void whiteListAccessFromOrigin(JSStringRef sourceOrigin, JSStringRef destinationProtocol, JSStringRef destinationHost, bool allowDestinationSubdomains);
+
+    void addUserScript(JSStringRef source, bool runAtStart);
+    void addUserStyleSheet(JSStringRef source);
+
+    void setGeolocationPermission(bool allow);
+    bool isGeolocationPermissionSet() const { return m_isGeolocationPermissionSet; }
+    bool geolocationPermission() const { return m_geolocationPermission; }
+
 private:
     bool m_dumpAsText;
     bool m_dumpAsPDF;
@@ -184,6 +218,7 @@ private:
     bool m_dumpEditingCallbacks;
     bool m_dumpResourceLoadCallbacks;
     bool m_dumpResourceResponseMIMETypes;
+    bool m_dumpWillCacheResponse;
     bool m_dumpFrameLoadCallbacks;
     bool m_callCloseOnWebViews;
     bool m_canOpenWindows;
@@ -195,12 +230,20 @@ private:
     bool m_waitToDump; // True if waitUntilDone() has been called, but notifyDone() has not yet been called.
     bool m_willSendRequestReturnsNullOnRedirect;
     bool m_windowIsKey;
-
+    bool m_alwaysAcceptCookies;
     bool m_globalFlag;
+    bool m_isGeolocationPermissionSet;
+    bool m_geolocationPermission;
+    bool m_handlesAuthenticationChallenges;
 
+    std::string m_authenticationUsername;
+    std::string m_authenticationPassword; 
     std::string m_testPathOrURL;
     std::string m_expectedPixelHash;    // empty string if no hash
     
+    // origins which have been granted desktop notification access
+    std::vector<JSStringRef> m_desktopNotificationAllowedOrigins;
+    
     static JSClassRef getJSClass();
     static JSStaticValue* staticValues();
     static JSStaticFunction* staticFunctions();
