@@ -31,6 +31,8 @@
 #include "config.h"
 #include "DumpRenderTree.h"
 
+#include "AccessibilityController.h"
+#include "EventSender.h"
 #include "GCController.h"
 #include "LayoutTestController.h"
 #include "WorkQueue.h"
@@ -61,6 +63,7 @@ extern void webkit_web_settings_add_extra_plugin_directory(WebKitWebView* view,
 extern gchar* webkit_web_frame_get_response_mime_type(WebKitWebFrame* frame);
 extern void webkit_web_frame_clear_main_frame_name(WebKitWebFrame* frame);
 extern void webkit_web_view_set_group_name(WebKitWebView* view, const gchar* groupName);
+extern void webkit_reset_origin_access_white_lists();
 }
 
 volatile bool done;
@@ -68,6 +71,7 @@ static bool printSeparators;
 static int dumpPixels;
 static int dumpTree = 1;
 
+AccessibilityController* axController = 0;
 LayoutTestController* gLayoutTestController = 0;
 static GCController* gcController = 0;
 static WebKitWebView* webView;
@@ -260,7 +264,7 @@ static void invalidateAnyPreviousWaitToDumpWatchdog()
     waitForPolicy = false;
 }
 
-static void resetWebViewToConsistentStateBeforeTesting()
+static void resetDefaultsToConsistentValues()
 {
     WebKitWebSettings* settings = webkit_web_view_get_settings(webView);
     g_object_set(G_OBJECT(settings),
@@ -271,12 +275,24 @@ static void resetWebViewToConsistentStateBeforeTesting()
                  "enable-html5-local-storage", TRUE,
                  "enable-xss-auditor", FALSE,
                  "javascript-can-open-windows-automatically", TRUE,
+                 "enable-offline-web-application-cache", TRUE,
+                 "enable-universal-access-from-file-uris", TRUE,
+                 "enable-scripts", TRUE,
+                 "default-font-family", "Times",
+                 "monospace-font-family", "Courier",
+                 "serif-font-family", "Times",
+                 "sans-serif-font-family", "Helvetica",
+                 "default-font-size", 16,
+                 "default-monospace-font-size", 13,
+                 "minimum-font-size", 1,
                  NULL);
 
     webkit_web_frame_clear_main_frame_name(mainFrame);
 
     WebKitWebInspector* inspector = webkit_web_view_get_inspector(webView);
     g_object_set(G_OBJECT(inspector), "javascript-profiling-enabled", FALSE, NULL);
+
+    webkit_reset_origin_access_white_lists();
 }
 
 void dump()
@@ -351,16 +367,7 @@ static void setDefaultsToConsistentStateValuesForTesting()
 {
     gdk_screen_set_resolution(gdk_screen_get_default(), 72.0);
 
-    WebKitWebSettings* settings = webkit_web_view_get_settings(webView);
-    g_object_set(G_OBJECT(settings),
-                 "default-font-family", "Times",
-                 "monospace-font-family", "Courier",
-                 "serif-font-family", "Times",
-                 "sans-serif-font-family", "Helvetica",
-                 "default-font-size", 16,
-                 "default-monospace-font-size", 13,
-                 "minimum-font-size", 1,
-                 NULL);
+    resetDefaultsToConsistentValues();
 
     /* Disable the default auth dialog for testing */
     SoupSession* session = webkit_get_default_session();
@@ -369,6 +376,10 @@ static void setDefaultsToConsistentStateValuesForTesting()
 #if PLATFORM(X11)
     webkit_web_settings_add_extra_plugin_directory(webView, TEST_PLUGIN_DIR);
 #endif
+
+    gchar* databaseDirectory = g_build_filename(g_get_user_data_dir(), "gtkwebkitdrt", "databases", NULL);
+    webkit_set_web_database_directory_path(databaseDirectory);
+    g_free(databaseDirectory);
 }
 
 static void runTest(const string& testPathOrURL)
@@ -388,7 +399,7 @@ static void runTest(const string& testPathOrURL)
     gchar* url = autocorrectURL(pathOrURL.c_str());
     const string testURL(url);
 
-    resetWebViewToConsistentStateBeforeTesting();
+    resetDefaultsToConsistentValues();
 
     gLayoutTestController = new LayoutTestController(testURL, expectedPixelHash);
     topLoadingFrame = 0;
@@ -404,6 +415,7 @@ static void runTest(const string& testPathOrURL)
 
     bool isSVGW3CTest = (gLayoutTestController->testPathOrURL().find("svg/W3C-SVG-1.1") != string::npos);
     GtkAllocation size;
+    size.x = size.y = 0;
     size.width = isSVGW3CTest ? 480 : maxViewWidth;
     size.height = isSVGW3CTest ? 360 : maxViewHeight;
     gtk_widget_size_allocate(container, &size);
@@ -415,7 +427,8 @@ static void runTest(const string& testPathOrURL)
     if (prevTestBFItem)
         g_object_ref(prevTestBFItem);
 
-
+    // Focus the web view before loading the test to avoid focusing problems
+    gtk_widget_grab_focus(GTK_WIDGET(webView));
     webkit_web_view_open(webView, url);
 
     g_free(url);
@@ -478,13 +491,21 @@ static void webViewLoadFinished(WebKitWebView* view, WebKitWebFrame* frame, void
 static void webViewWindowObjectCleared(WebKitWebView* view, WebKitWebFrame* frame, JSGlobalContextRef context, JSObjectRef windowObject, gpointer data)
 {
     JSValueRef exception = 0;
-    assert(gLayoutTestController);
+    ASSERT(gLayoutTestController);
 
     gLayoutTestController->makeWindowObject(context, windowObject, &exception);
-    assert(!exception);
+    ASSERT(!exception);
 
     gcController->makeWindowObject(context, windowObject, &exception);
     ASSERT(!exception);
+
+    axController->makeWindowObject(context, windowObject, &exception);
+    ASSERT(!exception);
+
+    JSStringRef eventSenderStr = JSStringCreateWithUTF8CString("eventSender");
+    JSValueRef eventSender = makeEventSender(context);
+    JSObjectSetProperty(context, windowObject, eventSenderStr, eventSender, kJSPropertyAttributeReadOnly | kJSPropertyAttributeDontDelete, 0);
+    JSStringRelease(eventSenderStr);
 }
 
 static gboolean webViewConsoleMessage(WebKitWebView* view, const gchar* message, unsigned int line, const gchar* sourceId, gpointer data)
@@ -586,6 +607,23 @@ static gboolean webViewClose(WebKitWebView* view)
     return TRUE;
 }
 
+static void databaseQuotaExceeded(WebKitWebView* view, WebKitWebFrame* frame, WebKitWebDatabase *database)
+{
+    ASSERT(view);
+    ASSERT(frame);
+    ASSERT(database);
+
+    WebKitSecurityOrigin* origin = webkit_web_database_get_security_origin(database);
+    if (gLayoutTestController->dumpDatabaseCallbacks()) {
+        printf("UI DELEGATE DATABASE CALLBACK: exceededDatabaseQuotaForSecurityOrigin:{%s, %s, %i} database:%s\n",
+            webkit_security_origin_get_protocol(origin),
+            webkit_security_origin_get_host(origin),
+            webkit_security_origin_get_port(origin),
+            webkit_web_database_get_name(database));
+    }
+    webkit_security_origin_set_web_database_quota(origin, 5 * 1024 * 1024);
+}
+
 
 static WebKitWebView* webViewCreate(WebKitWebView*, WebKitWebFrame*);
 
@@ -610,6 +648,7 @@ static WebKitWebView* createWebView()
                      "signal::status-bar-text-changed", webViewStatusBarTextChanged, 0,
                      "signal::create-web-view", webViewCreate, 0,
                      "signal::close-web-view", webViewClose, 0,
+                     "signal::database-quota-exceeded", databaseQuotaExceeded, 0,
                      NULL);
 
     return view;
@@ -665,6 +704,7 @@ int main(int argc, char* argv[])
     setDefaultsToConsistentStateValuesForTesting();
 
     gcController = new GCController();
+    axController = new AccessibilityController();
 
     if (argc == optind+1 && strcmp(argv[optind], "-") == 0) {
         char filenameBuffer[2048];
@@ -688,7 +728,10 @@ int main(int argc, char* argv[])
     delete gcController;
     gcController = 0;
 
-    g_object_unref(webView);
+    delete axController;
+    axController = 0;
+
+    gtk_widget_destroy(window);
 
     return 0;
 }
